// /************************************************************************\
// *                                                                        *
// *  Matt's SDL Library-ish v0.2                                           *
// *      by Mathew Carr                                                    *
// *                                                                        *
// \************************************************************************/

#include "mattsdl.h" // mattSDL!

// MSDL_event_callback_noop() :
//    Callback that does nothing. Default callback for the SDL events!
//
//    SDL_Event *sdl_event :
//       The event structure for this event. Too bad the callback does nothing.
void MSDL_event_callback_noop(SDL_Event *sdl_event) {
   // Do nothing!
}

// Array of flags telling what events have taken place.
bool MSDL_triggered_event_flags[SDL_NUMEVENTS];

// Array of callbacks for capturing the different events.
void (*MSDL_event_callbacks[SDL_NUMEVENTS])(SDL_Event *sdl_event) = {
   MSDL_event_callback_noop,
   MSDL_event_callback_noop,
   MSDL_event_callback_noop,
   MSDL_event_callback_noop,
   MSDL_event_callback_noop,

   MSDL_event_callback_noop,
   MSDL_event_callback_noop,
   MSDL_event_callback_noop,
   MSDL_event_callback_noop,
   MSDL_event_callback_noop,

   MSDL_event_callback_noop,
   MSDL_event_callback_noop,
   MSDL_event_callback_noop,
   MSDL_event_callback_noop,
   MSDL_event_callback_noop,

   MSDL_event_callback_noop,
   MSDL_event_callback_noop,
   MSDL_event_callback_noop,
   MSDL_event_callback_noop,
   MSDL_event_callback_noop,

   MSDL_event_callback_noop,
   MSDL_event_callback_noop,
   MSDL_event_callback_noop,
   MSDL_event_callback_noop,
   MSDL_event_callback_noop,

   MSDL_event_callback_noop,
   MSDL_event_callback_noop,
   MSDL_event_callback_noop,
   MSDL_event_callback_noop,
   MSDL_event_callback_noop,

   MSDL_event_callback_noop,
   MSDL_event_callback_noop,
};

// MSDL_UpdateEventsArray() :
//    Empties the SDL event queue, setting the MSDL_triggered_event_flags
//    array to true if the events were caught on this execution, and false
//    if they werent. This function executes the appropriate functions from
//    the array of callbacks MSDL_event_callbacks for each event caught.
//
//    bool MSDL_triggered_event_flags[] :
//       Array of bools. These will be set to true or false depending on
//       event was caught on this execution of the function.
//       If unspecified, this function will use the MSDL default.
//
//    void (*MSDL_event_callbacks[])(SDL_Event *sdl_event) :
//       Array of function pointers (taking SDL_Event *sdl_event parameter).
//       These callbacks will be executed with each event captured, passing the
//       event type responsible.
//       If not passed, the function will use the MSDL default.
//
void MSDL_UpdateEventsArray(bool MSDL_triggered_event_flags[],
                            void (*MSDL_event_callbacks[])(SDL_Event *sdl_event)) {
   SDL_PumpEvents();

   // Turn off all the flags.
   for (int event_type = 0; event_type < SDL_NUMEVENTS; event_type++) {
      MSDL_triggered_event_flags[event_type] = false;
   }

   // Structure to contain event information.
   SDL_Event sdl_event;

   // Check for program messages, store to sdl_event.
   while(SDL_PollEvent(&sdl_event)) {
      // Interpret messages.

      // Set flag for this event type.
      MSDL_triggered_event_flags[sdl_event.type] = true;

      // Launch the callback function for this particular event type.
      MSDL_event_callbacks[sdl_event.type](&sdl_event);
   }
}

// Pointer to all the keystates.
Uint8 MSDL_keystates[SDLK_LAST];
// Array to all the previous keystates.
Uint8 MSDL_prevkeystates[SDLK_LAST];
// Number of keys, returned from SDL_GetKeyState.
int MSDL_numkeys = 0;

// MSDL_GetKeyState() :
//    Wrapper for the SDL_GetKeyState function, storing the keystate in the
//    MSDL global MSDL_keystates array. This array won't fly about when you're not
//    looking like the SDL version!
void MSDL_GetKeyState() {
   // Copy all of the current keystates into the MSDL_prevkeystates array!
   for (int i = 0; i < MSDL_numkeys; i++) {
      MSDL_prevkeystates[i] = MSDL_keystates[i];
   }

   // Do the thing!
   Uint8 *newkeystates = SDL_GetKeyState(&MSDL_numkeys);

   // Copy all of the hyper-new keystates into the MSDL_keystates array!
   for (int i = 0; i < MSDL_numkeys; i++) {
      MSDL_keystates[i] = newkeystates[i];
   }
}

// All the current mousestates.
Uint8 MSDL_mousestates;
// All the previous mousestates.
Uint8 MSDL_prevmousestates;
// Mouse position, returned from SDL_GetMouseState.
int MSDL_mousex = 0;
int MSDL_mousey = 0;

// MSDL_GetMouseState() :
//    Wrapper for the SDL_GetMouseState function, storing the mousestate in the
//    MSDL global MSDL_mousestates Uint8, unless otherwise stated.
//
//    Uint8 *MSDL_mousestates :
//       Pointer to Uint8 representing the state of mouse buttons.
//       You can use the MSDL default, or pass a pointer to a Uint8 to store the
//       value in.
void MSDL_GetMouseState(Uint8 *MSDL_mousestates, int *MSDL_mousex, int *MSDL_mousey) {
   // Copy the current mousestate into MSDL_prevmousestates!
   MSDL_prevmousestates = (*MSDL_mousestates);

   // Do the thing!
   *MSDL_mousestates = SDL_GetMouseState(MSDL_mousex, MSDL_mousey);
}

// MSDL_KeyHeld() :
//    Returns 1 if the specified key was down during the last two calls to
//    MSDL_GetKeyState().
// MSDL_KeyPressed() :
//    Returns 1 if the specified key has been pressed in the last call to
//    MSDL_GetKeyState().
// MSDL_KeyRaised() :
//    Returns 1 if the specified key has been released in the last call to
//    MSDL_GetKeyState().
// MSDL_KeyIdle() :
//    Returns 1 if the specified key hasn't been pressed in the last call to
//    MSDL_GetKeyState().
//
//    int key_index :
//       The number of the key to test for... it should be an SDLK_* constant.
int MSDL_KeyHeld(int key_index) {
   return ( MSDL_keystates[key_index] &  MSDL_prevkeystates[key_index]);
}
int MSDL_KeyPressed(int key_index) {
   return ( MSDL_keystates[key_index] & !MSDL_prevkeystates[key_index]);
}
int MSDL_KeyRaised(int key_index) {
   return (!MSDL_keystates[key_index] &  MSDL_prevkeystates[key_index]);
}
int MSDL_KeyIdle(int key_index) {
   return (!MSDL_keystates[key_index] & !MSDL_prevkeystates[key_index]);
}

// MSDL_Delay() :
//   SDL_Delay style wait function. SDL's probably does the same, but just to be
//   sure... It's a classic spinlock delay.
//
//   Uint32 time :
//      The time in milliseconds for the function to spinlock for.
void MSDL_Delay(Uint32 time) {
   // Set the stopwatch now.
   MSDL_SetStartTime();

   // Wait until the stopwatch hits time.
   MSDL_SpinlockUntilStopwatch(time);
}

// The time the stopwatch was started. (This time is counted as zero)
Uint32 MSDL_start_time;

// MSDL_SetStartTime() :
//    Sets the internal stopwatch zero to this time now.
void MSDL_SetStartTime(void) {
   MSDL_start_time = SDL_GetTicks();
}

// MSDL_SpinlockUntilStopwatch() :
//   Waits until the stopwatch hits the time stopwatch_time.
//
//   Uint32 stopwatch_time :
//      The time in milliseconds for the stopwatch to hit until spinlock ends.
void MSDL_SpinlockUntilStopwatch(Uint32 stopwatch_time) {
   Uint32 destination = MSDL_start_time + stopwatch_time;

   while (SDL_GetTicks() < destination) {
      // The classic spinlock!
   }
}

