// Matrix2DInterface is an interface implemented by a number of
// classes abstracting a grid of numbers. It allows you to select
// a number from a grid, basically. However, the underlying implementation
// of the grid may be a non-Cartesian mapping, or it may wrap, etc.

#ifndef __MATRIX2DINTERFACE_H__
#define __MATRIX2DINTERFACE_H__

template <class T>
class Matrix2DInterface
{
   public:
      virtual       T &Element(int _x_c, int _y_c) = 0;
      virtual const T &Element(int _x_c, int _y_c) const = 0;

      virtual unsigned int Width() const = 0;
      virtual unsigned int Height() const = 0;

      virtual ~Matrix2DInterface()
      {
      }

      virtual void SetAll(const T &proto) = 0;

      // This does a function of void function_name(T &blah)
      // to all the elements in the Matrix2DInterface!
      virtual void ForEachUnary(void (*function)(T &, const void *context), const void *context)
      {
         const unsigned int width = Width(), height = Height();

         for (unsigned int y = 0; y < height; y++)
         {
            for (unsigned int x = 0; x < width; x++)
            {
               function(Element(x, y), context);
            }
         }
      }
   private:
   ;
};

#endif // __MATRIX2DINTERFACE_H__
