#ifndef __WIZARD_BRACELET_FONT_H__
#define __WIZARD_BRACELET_FONT_H__

#include "Colour.h"

#include <cstdlib>
#include <cstring>

namespace WizardBraceletFont
{
   bool Initialise();
   void Deinitialise();

   extern const float DEFAULT_CHARACTER_SIZE;
   extern const float DEFAULT_CHARACTER_SPACING;
   extern const float DEFAULT_LINE_SPACING;

   // Renders the given null terminated string to the position.
   // \n and \r are treated as newlines.

   // It fiddles with the render states and texture states.
   // Make sure to set these explicitly later before
   // attempting further drawing!
   void DrawString(const char *characters,
                   float l_x,                                                      // X position of left side of string.
                   float t_y,                                                      // Y position of top of string.
                   float x_size              = DEFAULT_CHARACTER_SIZE,             // X size of rendered character.
                   float y_size              = DEFAULT_CHARACTER_SIZE,             // Y size of rendered character.
                   COLOUR colour             = COLOUR_WHITE,                       // Colour to apply to text.
                   float x_character_spacing = DEFAULT_CHARACTER_SPACING,          // Spacing between characters. (in multiples of character size)
                   float y_line_spacing      = DEFAULT_LINE_SPACING);

   // Tries to draw a string centered around your coordinates.
   // This does NOT work with 'multiple line' strings.

   // As an acceleration, the user can supply the length of the string
   // if it is known beforehand to avoid the use of strlen, which
   // may be slow.
   static inline void DrawStringCenteredCS(const char *characters, size_t known_strlen,
                             float l_x,                                                 // X position of centre of string.
                             float t_y,                                                 // Y position of top of string.
                             float x_size              = DEFAULT_CHARACTER_SIZE,        // X size of rendered character.
                             float y_size              = DEFAULT_CHARACTER_SIZE,        // Y size of rendered character.
                             COLOUR colour             = COLOUR_WHITE,                  // Colour to apply to text.
                             float x_character_spacing = DEFAULT_CHARACTER_SPACING)     // Spacing between characters. (in multiples of character size)
   {
      // Move the string to the left so that it appears horizontally centered
      // at the given position.
      float new_x = l_x - (((((float)(known_strlen - 1)) * (x_character_spacing * x_size)) + x_size) / 2.0f);
      DrawString(characters, new_x, t_y, x_size, y_size, colour, x_character_spacing);
   }

   // Renders the given null terminated string to the position.
   // A wrapper function for DrawString.
   static inline void DrawStringCentered(const char *characters,
                             float l_x,                                                 // X position of centre of string.
                             float t_y,                                                 // Y position of top of string.
                             float x_size              = DEFAULT_CHARACTER_SIZE,        // X size of rendered character.
                             float y_size              = DEFAULT_CHARACTER_SIZE,        // Y size of rendered character.
                             COLOUR colour             = COLOUR_WHITE,                  // Colour to apply to text.
                             float x_character_spacing = DEFAULT_CHARACTER_SPACING)     // Spacing between characters. (in multiples of character size)
   {
      // Wrap further.
      DrawStringCenteredCS(characters, strlen(characters), l_x, t_y, x_size, y_size, colour, x_character_spacing);
   }
};

#endif // __WIZARD_BRACELET_FONT_H__
