// WrappingArray2D is a template class designed to hold a two dimensional
// array of T. It contains an accessor function that allows the user to
// grab an element, as if the array were mapped onto a repeating plane.

#ifndef __WRAPPINGARRAY2D_H__
#define __WRAPPINGARRAY2D_H__

#include "Matrix2DInterface.h"

#include <vector>

template <class T>
class WrappingArray2D : public Matrix2DInterface<T>
{
   public:
      // Constructor creates a wrapping array of a given size.
      WrappingArray2D(unsigned int _data_x_l, unsigned int _data_y_l)
      {
         assert(_data_x_l != 0);
         assert(_data_y_l != 0);

         data_x_l = _data_x_l;
         data_y_l = _data_y_l;

         // Insert data_x_l * data_y_l copies of standard T constructor.
         data.insert(data.begin(), _DataBlockSize(), T());
      }

      // Return element at x_c, y_c on plane.
      virtual T &Element(int _x_c, int _y_c)
      {
         int real_x_c = _x_c % data_x_l;
         int real_y_c = _y_c % data_y_l;

         return _ElementAt((unsigned int)real_x_c, (unsigned int)real_y_c);
      }

      virtual const T &Element(int _x_c, int _y_c) const
      {
         int real_x_c = _x_c % data_x_l;
         int real_y_c = _y_c % data_y_l;

         return _ElementAt((unsigned int)real_x_c, (unsigned int)real_y_c);
      }

      virtual unsigned int Width() const
      {
         return data_x_l;
      }

      virtual unsigned int Height() const
      {
         return data_y_l;
      }

      virtual void SetAll(const T &proto)
      {
         _SetAll(proto);
      }

      virtual ~WrappingArray2D()
      {
      }

      virtual void ForEachUnary(void (*f)(T &a))
      {

      }

   private:
      unsigned int data_x_l, data_y_l;

      unsigned int _DataBlockSize() const
      {
         return data_x_l * data_y_l;
      }

      // Storage type.
      std::vector<T> data;

      // Returns reference to element within valid grid.
      T &_ElementAt(unsigned int _x_c, unsigned int _y_c)
      {
         unsigned int element = _x_c + (_y_c * data_x_l);

         assert(element < _DataBlockSize());
         assert(element < data.size());

         return data.at(element);
      }

      // Returns reference to element within valid grid.
      const T &_ElementAt(unsigned int _x_c, unsigned int _y_c) const
      {
         unsigned int element = _x_c + (_y_c * data_x_l);

         assert(element < _DataBlockSize());
         assert(element < data.size());

         return data.at(element);
      }

      void _SetAll(const T &proto)
      {
         data.clear();

         data.insert(data.begin(), _DataBlockSize(), T(proto));
      }
};

#endif // __WRAPPINGARRAY2D_H__
