// TGA Loading Code By Mathew Carr
// Based on TGA Analysis by Trent 'ShiningKnight' Polack

#include "skMrDTGA.h"

#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <cmath>
#include <gl\gl.h>
#include <gl\glu.h>

void MessageBox(int a, const char *b, const char *c, int d)
{ /* Null it out*/ }

bool SK_MrD_LoadTGA(const char *filename, GLuint &texid) {   // Creates Texture From A Bitmap File

   const unsigned char  h_uncom[12] = {0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0}; // Uncompressed header
   const unsigned char    h_com[12] = {0, 0,10, 0, 0, 0, 0, 0, 0, 0, 0, 0}; // Compressed header

   unsigned char acidtest[12] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}; // Buffer to store TGA header.

   unsigned char infoheader[6]; // Buffer to store the TGA header info bytes

   int width, height, bitspp, bytespp, imagesz;

   unsigned char *data; // Pointer to loaded data.

   FILE *source_file; // Declare File Pointer
   source_file = fopen(filename, "rb"); // Open File For Reading

   // Fail if file pointer returned was NULL
   if (source_file == NULL) {
      MessageBox(NULL, "Couldn't get an fread() lockon!", "Error!", 0);
      MessageBox(NULL, filename, "Does this file even exist?", 0);
      return false;
   }

   // Fail if reading of first twelve bytes failed
   if (fread(acidtest, 12, 1, source_file) != 1) {
      MessageBox(NULL, "Couldn't read the twelve byte identifier!", "Error!", 0);
      return false;
   }

   if (memcmp(h_uncom, acidtest, 12) == 0) {
      // If the header showed this was an uncompressed TGA
      if(fread(infoheader, 6, 1, source_file) == 0) {
         MessageBox(NULL, "Couldn't read the header six bytes!", "Error!", 0);
         return false;
      }
      // Decode info header information:
      width   = infoheader[0] + infoheader[1] * 256;
      height  = infoheader[2] + infoheader[3] * 256;
      bitspp  = infoheader[4];
      bytespp = bitspp / 8;

      imagesz = width * height * bytespp;

      data    = (unsigned char *)malloc(imagesz);

      // Fail if image data could not be loaded
      if(fread(data, imagesz, 1, source_file) == 0) {
         MessageBox(NULL, "Couldn't read the image data itself!", "Error!", 0);
         return false;
      }

      // Swap colour channel information.
      for (long o = 0; o < imagesz; o += bytespp) {
         data[o] ^= data[o+2] ^= data[o] ^= data[o+2];      // swap those BGR into RGB.
      }

      fclose(source_file);

      glGenTextures(1, &texid);

      glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

      glBindTexture(GL_TEXTURE_2D, texid);                                // Bind To The Texture ID
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);   // Linear Min Filter
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);   // Linear Mag Filter
      glTexImage2D(GL_TEXTURE_2D, 0, bytespp, width, height, 0, (bitspp == 24) ? GL_RGB : GL_RGBA, GL_UNSIGNED_BYTE, data);
      free(data);
      return true;                                           // Loading Was Successful
   } else if (memcmp(h_com, acidtest, 12) == 0) {
      // If the header showed this was a compressed TGA
      if(fread(infoheader, 6, 1, source_file) == 0) {
         MessageBox(NULL, "Couldn't read the header six bytes!", "Error!", 0);
         return false;
      }
      // Decode header information
      width   = infoheader[0] + infoheader[1] * 256;
      height  = infoheader[2] + infoheader[3] * 256;
      bitspp  = infoheader[4];
      bytespp = bitspp / 8;

      imagesz = width * height * bytespp;

      data    = (unsigned char *)malloc(imagesz);

      long o  = 0;
      unsigned char chunkheader;
      unsigned char pixelbuffer[bytespp];

      // Decompress image by loading chunks and decoding them
      // until image is complete
      while (o < imagesz) {
         if (fread(&chunkheader, 1, 1, source_file) == 0) {
            MessageBox(NULL, "Couldn't read the chunkheader!", "Error!", 0);
            return true;
         }

         if (chunkheader < 128) {
            unsigned char pixelstored = chunkheader + 1;
            if (fread(&data[o], bytespp, pixelstored, source_file) != pixelstored) {
               MessageBox(NULL, "Couldn't get the (chunkheader+1)'s share of RAW pixels!", "Error!", 0);
               return true;
            }
            o += pixelstored*bytespp;
         } else {
            unsigned char pixelstoblast = chunkheader - 127;
            if (fread(pixelbuffer, bytespp, 1, source_file) != 1) {
               MessageBox(NULL, "Couldn't get the RAW pixel for an RLE blast!", "Error!", 0);
               return true;
            }
            for (int c = 0; c < pixelstoblast; c++) {
                                 data[o+0] = pixelbuffer[0];
                                 data[o+1] = pixelbuffer[1];
                                 data[o+2] = pixelbuffer[2];
               if (bytespp == 4) data[o+3] = pixelbuffer[3];
               o += bytespp;
            }
         }
      }

      // Swap colour channel information
      for (long o = 0; o < imagesz; o += bytespp) {
         data[o] ^= data[o+2] ^= data[o] ^= data[o+2];      // swap those BGR into RGB.
      }

      fclose(source_file);

      glGenTextures(1, &texid);

      glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

      glBindTexture(GL_TEXTURE_2D, texid);                                // Bind To The Texture ID
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);   // Linear Min Filter
      glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);   // Linear Mag Filter
      glTexImage2D(GL_TEXTURE_2D, 0, bytespp, width, height, 0, (bitspp == 24) ? GL_RGB : GL_RGBA, GL_UNSIGNED_BYTE, data);
      free(data);
      return true;                                           // Loading Was Successful
   } else {
      // The header was unrecognised
      MessageBox(NULL, "Can't get that TGA.", "Error!", 0);
      return false;
   }
}
