// Colour structure.

#ifndef __COLOUR_H__
#define __COLOUR_H__

enum COLOUR_CHANNEL
{
   R = 0,
   G = 1,
   B = 2,
   A = 3,
   NO_COLOUR_CHANNELS = 4,
};

struct COLOUR
{
   union
   {
      struct
      {
         float r;
         float g;
         float b;
         float a;
      };

      float values[NO_COLOUR_CHANNELS];
   };

   COLOUR(float _r, float _g, float _b, float _a = 1.0f)
     : r(_r), g(_g), b(_b), a(_a)
   {

   }

   COLOUR(float _grey = 1.0f)
     : r(_grey), g(_grey), b(_grey), a(1.0f)
   {

   }

   static COLOUR UC(unsigned char _r, unsigned char _g, unsigned char _b, unsigned char _a = 255)
   {
      return COLOUR((((float)_r) / 255.0f),
                    (((float)_g) / 255.0f),
                    (((float)_b) / 255.0f),
                    (((float)_a) / 255.0f));
   }
};

const COLOUR COLOUR_WHITE = COLOUR::UC((unsigned char)255, (unsigned char)255, (unsigned char)255);
const COLOUR COLOUR_BLACK = COLOUR::UC((unsigned char)  0, (unsigned char)  0, (unsigned char)  0);
const COLOUR COLOUR_RED   = COLOUR::UC((unsigned char)255, (unsigned char)  0, (unsigned char)  0);
const COLOUR COLOUR_GREEN = COLOUR::UC((unsigned char)  0, (unsigned char)255, (unsigned char)  0);
const COLOUR COLOUR_BLUE  = COLOUR::UC((unsigned char)  0, (unsigned char)  0, (unsigned char)255);

#endif // __COLOUR_H__
