#include <cstdlib>
#include <cstdio>
#include <ctime>

#include "mt19937ar.h"
#include "rngs.h"

inline double ansi_c_rand_wrapper()
{
   return ((double)rand()) / ((double)RAND_MAX);
}

inline float ansi_c_rand_wrapperf()
{
   return ((float)rand()) / ((float)RAND_MAX);
}

template <typename T>
void PerformQuantitativeRandomAnalysis(
   T (*random_function)(),              /* Function pointer to function producing Ts */
   unsigned int repetitions_per_cycle,  /* How many numbers to generate within a cycle */
   unsigned int cycles,                 /* How many cycles to run */
   float time_for_cycle[]               /* Output: Array of floats to store
                                           the time in seconds to run a cycle */
)
{
   clock_t time_start, time_end;

   /* Run each cycle. */
   for (unsigned int cycle = 0; cycle < cycles; cycle++)
   {
      time_start = clock();

      for (unsigned int repetition = 0; repetition < repetitions_per_cycle; repetition++)
      {
         random_function();
      }

      time_end = clock();

      time_for_cycle[cycle] = ((float)(time_end - time_start)) / ((float)CLOCKS_PER_SEC);
   }
}

int main(int argc, char *argv[])
{
   MT::init_genrand(time(NULL));
   RNGS::PutSeed(time(NULL));
   srand(time(NULL));

   const unsigned int NO_REPETITIONS = 20 * 1000 * 1000;
   const unsigned int NO_CYCLES = 30;

   double (*randfunction[3])();

   randfunction[0] = ansi_c_rand_wrapper;
   randfunction[1] = MT::genrand_real2;
   randfunction[2] = RNGS::Random;

//   float (*randfunction[3])();
//
//   randfunction[0] = ansi_c_rand_wrapperf;
//   randfunction[1] = MT::genrand_realf2;
//   randfunction[2] = RNGS::Randomf;

   for (int f = 0; f < 3; f++)
   {
      float outputs[NO_CYCLES];

      PerformQuantitativeRandomAnalysis<double>(randfunction[f], NO_REPETITIONS, NO_CYCLES, outputs);

      float total = 0.0f;

      for (unsigned int i = 0; i < NO_CYCLES; i++)
      {
         total += outputs[i];
         printf("%7.3fs ", outputs[i]);
         if (((i+1)%5) == 0) printf("\r\n");
      }

      printf("= %7.3fs for method %d \r\n\r\n", total, f+1);
   }

   return 0;
}
